@@ -1,15 +1,21 @@
 from MainWindow import MainWindow
 from SplashScreen import SplashScreen
 from RootPane import RootPane
-from ContentPane import ContentPane
+from TitlePanel import TitlePanel
+from ControlPanel import ControlPanel
+from TabPanel import TabPanel
+from WindowControls import WindowControls
 from Thumbnailer import Thumbnailer
 from ViewerState import ViewerState
 from viewers.Thumbnail import Thumbnail
+from ui.HBox import HBox
+from ui.VBox import VBox
+from ui.Image import Image
 from ui.Pixmap import Pixmap
 from ui.ImageStrip import ImageStrip
 from ui.KineticScroller import KineticScroller
+from ui import pixbuftools
 from ui import dialogs
-from ControlBar import ControlBar
 import panel_actions
 from Headset import Headset
 from mediascanner.MediaScanner import MediaScanner
@@ -25,6 +31,12 @@ import os
 import time
 
 
+_MODE_NORMAL = 0
+_MODE_NO_STRIP = 1
+_MODE_FULLSCREEN = 2
+_MODE_TITLE_ONLY = 3
+
+
 class App(object):
     """
     Main class of the application.
@@ -36,6 +48,7 @@ class App(object):
         self.__current_viewer = None
         self.__current_collection = []
         self.__current_mediaroots = []
+        self.__view_mode = _MODE_TITLE_ONLY
         
         # mapping: viewer -> state
         self.__viewer_states = {}
@@ -46,6 +59,11 @@ class App(object):
         # flag for indicating whether media scanning has already been scheduled
         self.__media_scan_scheduled = False
     
+        # search string for finding items with the keyboard
+        self.__keyboard_search_string = ""
+        # time after which the keyboard search string gets reset
+        self.__keyboard_search_reset_time = 0
+    
         if (maemo.IS_MAEMO):
             import hildon
             import osso
@@ -59,8 +77,7 @@ class App(object):
         except:
             # theme could not be loaded
             pass
-            
-    
+
         # window
         self.__window = MainWindow()
         self.__window.set_app_paintable(True)
@@ -75,7 +92,7 @@ class App(object):
 
         # screen pixmap
         self.__screen = Pixmap(self.__window.window)
-        self.__screen.draw_pixbuf(theme.background, 0, 0)
+        #self.__screen.draw_pixbuf(theme.background, 0, 0)
 
         # root pane
         self.__root_pane = RootPane(self.__window)
@@ -86,57 +103,89 @@ class App(object):
         self.__splash.set_visible(True)
         self.__root_pane.add(self.__splash)
 
-        # thumbnail screen
-        self.__thumbnailer = Thumbnailer(self.__window)
-        self.__thumbnailer.set_visible(False)
-        self.__root_pane.add(self.__thumbnailer)
-        
-        # content pane
-        self.__content_pane = ContentPane(self.__window)
-        self.__root_pane.add(self.__content_pane)
-        self.__content_pane.set_visible(False)
-        
-        # control bar
-        self.__ctrlbar = ControlBar(self.__window)
-        self.__ctrlbar.set_pos(0, 400)
-        self.__ctrlbar.add_observer(self.__on_observe_ctrlbar)
-        self.__root_pane.add(self.__ctrlbar)
-       
-        # image strip
-        self.__strip = ImageStrip(self.__window, 160, 120, 10)
-        self.__strip.set_pos(10, 0)
-        self.__strip.set_size(160, 400)
-        self.__strip.set_background(theme.background.subpixbuf(10, 0, 160, 400))
-        self.__content_pane.add(self.__strip)
+        left_top = pixbuftools.make_frame(theme.panel, 170, 40, True,
+                                          pixbuftools.LEFT |
+                                          pixbuftools.BOTTOM)
+        left_bottom = pixbuftools.make_frame(theme.panel, 170, 70, True,
+                                          pixbuftools.TOP |
+                                          pixbuftools.LEFT)
+        pixbuftools.draw_pbuf(left_bottom, theme.btn_turn_1, 30, 15)
+
+
+        # image strip       
+        self.__strip = ImageStrip(self.__window, 10)
+        self.__strip.set_geometry(0, 0, 170, 480)
+        #self.__strip.set_caps(theme.panel_top_left, theme.panel_left)
+        self.__strip.set_caps(left_top, left_bottom)
+        self.__strip.set_bg_color(theme.color_bg)
+        self.__strip.set_visible(False)
+        self.__root_pane.add(self.__strip)
 
         self.__kscr = KineticScroller(self.__strip)
         self.__kscr.set_touch_area(0, 108)
         self.__kscr.add_observer(self.__on_observe_strip)
 
+        # thumbnail screen
+        self.__thumbnailer = Thumbnailer(self.__window)
+        self.__thumbnailer.set_visible(False)
+        self.__root_pane.add(self.__thumbnailer)
+
+        # title panel
+        self.__title_panel_left = Image(self.__window, left_top)
+        self.__title_panel_left.set_geometry(0, 0, 170, 40)
+        self.__title_panel_left.set_visible(False)
+        
+        self.__title_panel = TitlePanel(self.__window)
+        self.__title_panel.set_geometry(170, 0, 630, 40)
+        self.__title_panel.set_title("Initializing")
+        self.__title_panel.set_visible(False)       
+       
+        # control panel
+        self.__panel_left = Image(self.__window, left_bottom)
+        self.__panel_left.set_geometry(0, 410, 160, 70)
+        self.__panel_left.connect(self.__panel_left.EVENT_BUTTON_PRESS,
+                                  self.__on_menu_button)
+        self.__panel_left.set_visible(False)
+        
+        self.__ctrl_panel = ControlPanel(self.__window)
+        self.__ctrl_panel.set_geometry(170, 410, 630, 70)
+        self.__ctrl_panel.set_bg(theme.panel)
+        self.__ctrl_panel.add_observer(self.__on_observe_ctrlbar)
+        self.__ctrl_panel.set_visible(False)
+
+        # tab panel and window controls
+        self.__tab_panel = TabPanel(self.__window)
+        self.__tab_panel.set_geometry(0, 330, 800, 150)
+        self.__tab_panel.set_visible(False)
+        self.__tab_panel.add_observer(self.__on_observe_tabs)
+        
+        self.__window_ctrls = WindowControls(self.__window)
+        self.__window_ctrls.set_geometry(600, 0, 200, 80)
+        self.__window_ctrls.set_visible(False)
+        self.__window_ctrls.add_observer(self.__on_observe_window_ctrls)
+        
+
         # set up media scanner
         mscanner = MediaScanner()
         mscanner.add_observer(self.__on_observe_media_scanner)
-      
-        # watch headset button
-        Headset().add_observer(self.__on_observe_headset)
-        
+             
        
     def __startup(self):
         """
         Runs a queue of actions to take for startup.
         """
         
-        actions = [(self.__root_pane.render, []),
+        actions = [(self.__root_pane.render_buffered, []),
                    (self.__load_viewers, []),
-                   (self.__check_for_player, []),
-                   #(time.sleep, [500]),
-                   (self.__splash.set_visible, [False]),
-                   (self.__content_pane.set_visible, [True]),
-                   (self.__ctrlbar.set_visible, [True]),                   
-                   (self.__root_pane.render, []),
+                   (self.__root_pane.add, [self.__tab_panel]),
+                   (self.__root_pane.add, [self.__window_ctrls]),                   
+                   (self.__add_panels, []),
+                   (self.__splash.set_visible, [False]),                   
+                   (self.__root_pane.render_buffered, []),
                    (self.__scan_media, [True]),
-                   (self.__ctrlbar.select_tab, [1]),
+                   (self.__select_viewer, [0]),
                    (self.__setup_mmc_replacement_detection, []),
+                   (Headset().add_observer, [self.__on_observe_headset]),
                    ]
                    
         def f():
@@ -145,14 +194,71 @@ class App(object):
                 try:
                     act(*args)
                 except:
+                    import traceback; traceback.print_exc()
                     pass
                 return True
             else:
                 return False
                 
         gobject.idle_add(f)
-        
 
+
+    def __add_panels(self):
+        """
+        Adds the panel components.
+        """
+    
+        self.__root_pane.add(self.__title_panel_left)
+        self.__root_pane.add(self.__title_panel)
+        self.__root_pane.add(self.__panel_left)
+        self.__root_pane.add(self.__ctrl_panel)
+        
+        
+    def __set_view_mode(self, view_mode):
+        """
+        Sets the view mode.
+        """
+
+        if (view_mode == _MODE_NORMAL):
+            self.__title_panel_left.set_visible(False)
+            self.__title_panel.set_visible(True)
+            self.__panel_left.set_visible(False)
+            self.__ctrl_panel.set_visible(True)
+            self.__strip.set_visible(True)
+            if (self.__current_viewer):
+                self.__current_viewer.set_geometry(180, 0, 620, 480)
+            
+        elif (view_mode == _MODE_NO_STRIP):
+            self.__title_panel_left.set_visible(True)
+            self.__title_panel.set_visible(True)
+            self.__panel_left.set_visible(True)
+            self.__ctrl_panel.set_visible(True)
+            self.__strip.set_visible(False)
+            if (self.__current_viewer):            
+                self.__current_viewer.set_geometry(0, 0, 800, 480)
+            
+        elif (view_mode == _MODE_FULLSCREEN):
+            self.__title_panel_left.set_visible(False)
+            self.__title_panel.set_visible(False)
+            self.__panel_left.set_visible(False)
+            self.__ctrl_panel.set_visible(False)
+            self.__strip.set_visible(False)
+            if (self.__current_viewer):            
+                self.__current_viewer.set_geometry(0, 0, 800, 480)
+
+        elif (view_mode == _MODE_TITLE_ONLY):
+            self.__title_panel_left.set_visible(True)
+            self.__title_panel.set_visible(True)
+            self.__panel_left.set_visible(False)
+            self.__ctrl_panel.set_visible(False)
+            self.__strip.set_visible(False)
+            if (self.__current_viewer):            
+                self.__current_viewer.set_geometry(0, 0, 800, 480)
+
+                
+        self.__view_mode = view_mode
+
+            
     def __setup_mmc_replacement_detection(self):
 
         import dbus, dbus.glib
@@ -174,6 +280,32 @@ class App(object):
             gobject.timeout_add(500, f)
 
 
+    def __on_menu_button(self, px, py):
+        """
+        Reacts on pressing the menu button.
+        """
+        
+        self.__show_tabs()
+
+
+    def __show_tabs(self):
+
+        self.__root_pane.set_enabled(False)
+        self.__root_pane.set_frozen(True)
+        
+        self.__tab_panel.set_enabled(True)
+        self.__tab_panel.set_frozen(False)
+        self.__tab_panel.set_visible(True)
+        
+        self.__tab_panel.fx_raise()
+
+        self.__window_ctrls.set_frozen(False)
+        self.__window_ctrls.set_enabled(True)        
+        self.__window_ctrls.set_visible(True)
+        self.__window_ctrls.fx_slide_in()
+
+
+
     def __scan_media(self, force_scan):
         """
         Scans the media root locations for media files. Will create thumbnails
@@ -198,11 +330,13 @@ class App(object):
         #end if
 
 
-        self.__ctrlbar.show_message("Looking for media files...")
-        self.__content_pane.set_visible(False)
-        self.__thumbnailer.clear()
+        view_mode = self.__view_mode
+        self.__set_view_mode(_MODE_TITLE_ONLY)
+        if (self.__current_viewer):
+            self.__current_viewer.set_visible(False)
+        self.__thumbnailer.clear()        
         self.__thumbnailer.set_visible(True)
-        self.__root_pane.render()
+        self.__root_pane.render_buffered()
         while (gtk.events_pending()): gtk.main_iteration()
 
 
@@ -217,7 +351,6 @@ class App(object):
         #self.__ctrlbar.show_message("Took %f seconds" % (time.time() - now))
         #while (gtk.events_pending()): gtk.main_iteration()
 
-        self.__ctrlbar.show_message("Generating Thumbnails...")
         while (gtk.events_pending()): gtk.main_iteration()
 
         # update viewers        
@@ -228,12 +361,13 @@ class App(object):
             self.__viewer_states[v].selected_item = -1
             self.__viewer_states[v].item_offset = 0
 
+        self.__set_view_mode(view_mode)
+        if (self.__current_viewer):
+            self.__current_viewer.set_visible(True)        
         self.__thumbnailer.set_visible(False)
-        self.__content_pane.set_visible(True)
-        self.__root_pane.render()
+        self.__root_pane.render_buffered()
 
         import gc; gc.collect()
-        self.__ctrlbar.show_panel()                
         
 
 
@@ -250,13 +384,13 @@ class App(object):
                 import traceback; traceback.print_exc()
                 continue
 
-            self.__ctrlbar.add_tab(viewer.ICON, viewer.ICON_ACTIVE, cnt)
+            self.__tab_panel.add_viewer(viewer)
+            #self.__ctrlbar.add_tab(viewer.ICON, viewer.ICON_ACTIVE, cnt)
             cnt += 1
 
-            self.__content_pane.add(viewer)
+            self.__root_pane.add(viewer)
             viewer.set_visible(False)
-            viewer.set_pos(180, 0)
-            viewer.set_size(620, 400)
+            #viewer.set_geometry(180, 0, 620, 480)
             viewer.add_observer(self.__on_observe_viewer)
             
             vstate = ViewerState()
@@ -265,21 +399,7 @@ class App(object):
             self.__viewers.append(viewer)
         #end for
                 
-        
-    def __check_for_player(self):
-        """
-        Checks if a media player backend is available and tells the user
-        if not.
-        """
-    
-        from mediaplayer.MPlayer import MPlayer
-        mplayer = MPlayer.MPlayer()
-        
-        if (not mplayer.is_available()):
-            dialogs.warning("mplayer was not found!",
-                 "Please install mplayer on your\ninternet tablet in order to\n"
-                 "be able to play video or audio.")
-                        
+                               
                         
     def __on_close_window(self, src, ev):
     
@@ -294,10 +414,22 @@ class App(object):
 
 
     def __on_key(self, src, ev):
+
+        # show memory consumption      
+        import os
+        pid = os.getpid()
+        size = int(open("/proc/%d/status" % pid, "r").read().splitlines()[15].split()[1])
+        size /= 1024.0
+        print "Current Resident Set Size: %0.02f MB" % size
+                
+    
+        if (not self.__root_pane.is_enabled()): return
     
         keyval = ev.keyval
         key = gtk.gdk.keyval_name(keyval)
         
+        if (key == "space"): key = " "
+        
         if (key == "Escape"):
             self.__try_quit()
         elif (key == "Return"):
@@ -313,6 +445,44 @@ class App(object):
             self.__kscr.impulse(0, 7.075)
         elif (key == "Down"):
             self.__kscr.impulse(0, -7.075)
+            
+        elif (key == "XF86Headset"):
+            self.__current_viewer.do_enter()
+        
+        elif (key == "BackSpace"):
+            now = time.time()        
+            if (now >= self.__keyboard_search_reset_time):
+                self.__keyboard_search_string = ""
+
+            if (self.__keyboard_search_string):
+                self.__keyboard_search_string = \
+                  self.__keyboard_search_string[:-1]
+
+            self.__keyboard_search_reset_time = now + 2
+
+            self.__title_panel.set_title_with_timeout("Search: " + \
+                                             self.__keyboard_search_string,
+                                             2000)
+
+            if (self.__keyboard_search_string):
+                self.__current_viewer.search(self.__keyboard_search_string)
+
+        
+        elif (len(key) == 1 and ord(key) > 31):
+            now = time.time()
+            if (now >= self.__keyboard_search_reset_time):
+                self.__keyboard_search_string = ""
+                
+            self.__keyboard_search_string += key.lower()
+            self.__keyboard_search_reset_time = now + 2
+
+            self.__title_panel.set_title_with_timeout("Search: " + \
+                                             self.__keyboard_search_string,
+                                             2000)
+
+            self.__current_viewer.search(self.__keyboard_search_string)
+            
+        
         else:
             print "unknown key", key
             
@@ -324,107 +494,151 @@ class App(object):
         if (cmd == src.OBS_THUMBNAIL_GENERATED):
             thumburi, uri = args
             name = os.path.basename(uri)
+            self.__title_panel.set_title(name)
             self.__thumbnailer.show_thumbnail(thumburi, name)
 
 
+    def __on_observe_window_ctrls(self, src, cmd, *args):
+    
+        if (cmd == src.OBS_MINIMIZE_WINDOW):
+            self.__window.iconify()
+            
+        elif (cmd == src.OBS_CLOSE_WINDOW):
+            self.__try_quit()
+
+
+    def __on_observe_tabs(self, src, cmd, *args):
+    
+        if (cmd == src.OBS_TAB_SELECTED):
+            idx = args[0]
+            #self.__tab_panel.fx_lower()
+
+            self.__window_ctrls.set_visible(False)
+            self.__window_ctrls.fx_slide_out()
+            self.__tab_panel.set_visible(False)
+
+            if (self.__viewers[idx] != self.__current_viewer):
+                #self.__root_pane.fx_fade_out()
+                self.__root_pane.set_enabled(True)
+                self.__root_pane.set_frozen(False)
+                self.__select_viewer(idx)
+            else:
+                self.__tab_panel.fx_lower()
+                self.__root_pane.set_enabled(True)
+                self.__root_pane.set_frozen(False)
+                self.__root_pane.render_buffered()
+
 
     def __on_observe_viewer(self, src, cmd, *args):
             
-        if (cmd == src.OBS_MINIMIZE):
-            self.__window.iconify()
+        #if (cmd == src.OBS_MINIMIZE):
+        #    self.__window.iconify()
             
-        elif (cmd == src.OBS_QUIT):
-            self.__try_quit()
+        #elif (cmd == src.OBS_QUIT):
+        #    self.__try_quit()
             
-        elif (cmd == src.OBS_REPORT_CAPABILITIES):
+        if (cmd == src.OBS_REPORT_CAPABILITIES):
             caps = args[0]
-            self.__ctrlbar.set_capabilities(caps)
-            self.__get_vstate().caps = caps
+            if (src.is_active() and caps != self.__get_vstate().caps):
+                self.__ctrl_panel.set_capabilities(caps)
+            self.__get_vstate(src).caps = caps
     
         elif (cmd == src.OBS_SCAN_MEDIA):
             force = args[0]
             self.__scan_media(force)
 
         elif (cmd == src.OBS_STATE_PLAYING):
-            self.__ctrlbar.set_playing(True)
+            if (src.is_active()):
+                self.__ctrl_panel.set_playing(True)
         
         elif (cmd == src.OBS_STATE_PAUSED):
-            self.__ctrlbar.set_playing(False)
+            if (src.is_active()):        
+                self.__ctrl_panel.set_playing(False)
 
         elif (cmd == src.OBS_TITLE):
             title = args[0]
-            self.__ctrlbar.set_title(title)
+            if (src.is_active()):
+                self.__title_panel.set_title(title)
+            self.__get_vstate(src).title = title
+    
+        elif (cmd == src.OBS_TIME):
+            pos, total = args
+            pos_m = pos / 60
+            pos_s = pos % 60
+            total_m = total / 60
+            total_s = total % 60
+            info = "%d:%02d / %d:%02d" % (pos_m, pos_s, total_m, total_s)
+            if (src.is_active()):
+                self.__title_panel.set_info(info)
+                self.__ctrl_panel.set_position(pos, total)
+            self.__get_vstate(src).info = info
     
         elif (cmd == src.OBS_POSITION):
             pos, total = args
-            self.__ctrlbar.set_position(pos, total)
+            info = "%d / %d" % (pos, total)
+            if (src.is_active()):
+                self.__title_panel.set_info(info)
+            self.__get_vstate(src).info = info
             
         elif (cmd == src.OBS_FREQUENCY_MHZ):
             freq = args[0]
-            unit = "MHz"
-            self.__ctrlbar.set_value(freq, unit)
+            info = "%03.2f MHz" % freq
+            if (src.is_active()):
+                self.__title_panel.set_info(info)
+            self.__get_vstate(src).info = info
             
         elif (cmd == src.OBS_VOLUME):
             percent = args[0]
-            self.__ctrlbar.set_volume(percent)
+            self.__title_panel.set_volume(percent)
             
         elif (cmd == src.OBS_SET_COLLECTION):
             items = args[0]
-            self.__current_collection = items
-            self.__set_collection(items)
+            if (src.is_active()):
+                self.__current_collection = items
+                self.__set_collection(items)
 
-            self.__saved_image = None
-            self.__saved_image_index = -1
+                self.__saved_image = None
+                self.__saved_image_index = -1
            
         elif (cmd == src.OBS_SELECT_ITEM):
             idx = args[0]
-            self.__select_item(idx)
+            if (src.is_active()):
+                self.__select_item(idx)
             
         elif (cmd == src.OBS_RENDER):
             self.__root_pane.render_buffered()
             
         elif (cmd == src.OBS_SHOW_COLLECTION):
-            self.__strip.set_visible(True)
-            #self.__strip.render()
-            #self.__strip.fx_slide_in()
-            self.__get_vstate().collection_visible = True
-            self.__current_viewer.set_pos(180, 0)
+            self.__set_view_mode(_MODE_NORMAL)
+            self.__get_vstate(src).collection_visible = True
 
         elif (cmd == src.OBS_HIDE_COLLECTION):
-            self.__strip.set_visible(False)
-            #self.__strip.fx_slide_out()
-            self.__get_vstate().collection_visible = False
-            self.__current_viewer.set_pos(0, 0)
+            self.__set_view_mode(_MODE_NO_STRIP)        
+            self.__get_vstate(src).collection_visible = False
 
         elif (cmd == src.OBS_FULLSCREEN):
-            self.__strip.set_visible(False)
-            self.__ctrlbar.set_visible(False)
-            self.__current_viewer.set_pos(0, 0)
-            self.__current_viewer.set_size(800, 480)
+            self.__set_view_mode(_MODE_FULLSCREEN)
 
         elif (cmd == src.OBS_UNFULLSCREEN):
-            self.__strip.set_visible(True)
-            self.__ctrlbar.set_visible(True)
-            self.__current_viewer.set_pos(180, 0)
-            self.__current_viewer.set_size(620, 400)
+            self.__set_view_mode(_MODE_NORMAL)
             
-        elif (cmd == src.OBS_SHOW_MESSAGE):
-            msg = args[0]
-            self.__ctrlbar.show_message(msg)
+        #elif (cmd == src.OBS_SHOW_MESSAGE):
+        #    msg = args[0]
+        #    #self.__ctrlbar.show_message(msg)
             
-        elif (cmd == src.OBS_SHOW_PROGRESS):
-            value, total = args
-            self.__ctrlbar.show_progress("Loading...", value, total)
+        #elif (cmd == src.OBS_SHOW_PROGRESS):
+        #    value, total = args
+        #    #self.__ctrlbar.show_progress("Loading...", value, total)
             
-        elif (cmd == src.OBS_SHOW_PANEL):
-            self.__ctrlbar.show_panel()
+        #elif (cmd == src.OBS_SHOW_PANEL):
+        #    pass #self.__ctrlbar.show_panel()
 
         elif (cmd == src.OBS_STOP_PLAYING):
             issuer = args[0]
             # call all viewers to stop playing
             for v in self.__viewers:
                 v.stop_playing(issuer)
-
+                
 
     def __on_observe_ctrlbar(self, src, cmd, *args):
     
@@ -454,6 +668,9 @@ class App(object):
     
         elif (cmd == panel_actions.FORCE_SPEAKER):
             self.__current_viewer.do_toggle_speaker()
+            
+        elif (cmd == panel_actions.TOGGLE_PLAYLIST):
+            self.__current_viewer.do_toggle_playlist()
     
         elif (cmd == panel_actions.SET_POSITION):
             pos = args[0]
@@ -462,24 +679,26 @@ class App(object):
         elif (cmd == panel_actions.TUNE):
             pos = args[0]
             self.__current_viewer.do_tune(pos)
-            
-        elif (cmd == panel_actions.TAB_SELECTED):
-            idx = args[0]
-            self.__select_viewer(idx)
-                                               
+
 
     def __on_observe_strip(self, src, cmd, *args):
     
+        handled = False
         if (cmd == src.OBS_SCROLLING):
             # invalidate ticket
             self.__ticket = 0
     
         elif (cmd == src.OBS_CLICKED):
             px, py = args
-            if (px > 120):
+            if (30 <= py < 420 and px > 108):
                 idx = self.__strip.get_index_at(py)            
                 self.__select_item(idx)
-           
+                handled = True
+            elif (0 <= px <= 80 and py >= 420):
+                self.__show_tabs()
+                handled = True
+
+        return handled
            
     def __on_observe_headset(self, src, cmd, *args):
     
@@ -487,9 +706,12 @@ class App(object):
             self.__current_viewer.do_enter()
            
            
-    def __get_vstate(self):
+    def __get_vstate(self, viewer = None):
     
-        return self.__viewer_states[self.__current_viewer]
+        if (not viewer):
+            viewer = self.__current_viewer
+        return self.__viewer_states[viewer]
+                
             
             
     def __select_item(self, idx):
@@ -504,28 +726,28 @@ class App(object):
 
     def __hilight_item(self, idx):
     
-        if (self.__saved_image):
-            self.__strip.replace_image(self.__saved_image_index,
-                                        self.__saved_image)
-        self.__saved_image = self.__strip.get_image(idx)
+        # restore saved image
+        if (self.__saved_image_index >= 0):
+            img = self.__strip.get_image(self.__saved_image_index)
+            img.set_hilighted(False)
+            #img.draw_pixmap(self.__saved_image, 0, 0)
+            #del self.__saved_image
+        
+        if (idx >= 0):    
+            img = self.__strip.get_image(idx)
+            img.set_hilighted(True)
+            #self.__saved_image = img.clone()
+            #img.draw_pixbuf(theme.selection_frame, 0, 0)
+        
         self.__saved_image_index = idx
-        
-        hilighted = self.__saved_image.copy()
-        theme.selection_frame.composite(hilighted, 0, 0,
-                                        hilighted.get_width(),
-                                        hilighted.get_height(),
-                                        0, 0, 1, 1, gtk.gdk.INTERP_NEAREST,
-                                        0xff)
-        
-        self.__strip.replace_image(idx, hilighted)
-        
+        self.__strip.render()        
 
             
     def __select_viewer(self, idx):
         """
         Selects the current viewer by index number.
         """        
-    
+      
         viewer = self.__viewers[idx]
         
         if (self.__current_viewer):
@@ -535,22 +757,19 @@ class App(object):
         self.__current_viewer = viewer
         self.__kscr.stop_scrolling()
 
-        buf = Pixmap(None, 800, 400)
-        self.__content_pane.set_screen(buf)
-
         vstate = self.__get_vstate()
         if (vstate.collection_visible):
-            viewer.set_pos(180, 0)
-            viewer.set_size(620, 400)
-            self.__strip.set_visible(True)
+            self.__set_view_mode(_MODE_NORMAL)
         else:
-            viewer.set_pos(0, 0)
-            viewer.set_size(800, 400)
-            self.__strip.set_visible(False)
+            self.__set_view_mode(_MODE_NO_STRIP)
         
         def f():
+            self.__root_pane.set_frozen(True)
             viewer.show()
-            self.__ctrlbar.set_capabilities(vstate.caps)
+            self.__ctrl_panel.set_capabilities(vstate.caps)
+            #self.__title_panel.set_time(0, 0)
+            self.__title_panel.set_title(vstate.title)
+            self.__title_panel.set_info(vstate.info)
             
             offset = vstate.item_offset
             item_idx = vstate.selected_item
@@ -558,9 +777,8 @@ class App(object):
             if (item_idx >= 0):
                 self.__hilight_item(item_idx)
 
-            self.__content_pane.render()
-            self.__content_pane.set_screen(self.__screen)
-            self.__content_pane.fx_raise(buf)
+            self.__root_pane.fx_slide_in() #render() #_buffered()
+            self.__root_pane.set_frozen(False)
 
         gobject.idle_add(f)
 
@@ -571,6 +789,7 @@ class App(object):
         Loads the given collection into the item strip.
         """
 
+        self.__hilight_item(-1)
         thumbnails = [ item.get_thumbnail() for item in collection ]                
         
         vstate = self.__get_vstate()        
@@ -578,12 +797,10 @@ class App(object):
             total = len(thumbnails)
             cnt = 1
             for t in thumbnails:
-                t.get_width()
+                if (cnt < 50): t.render()
 
-                if (cnt % 5 == 0 or (cnt == total and cnt > 5)):
-                    self.__ctrlbar.show_progress("Loading %d Items..." % total,
-                                                 cnt, total)                
-                    while (gtk.events_pending()): gtk.main_iteration()
+                #if (cnt % 5 == 0 or (cnt == total and cnt > 5)):
+                #    while (gtk.events_pending()): gtk.main_iteration()
                     
                 cnt += 1                    
             #end for
@@ -593,13 +810,20 @@ class App(object):
 
         if (not vstate.thumbs_loaded):
             vstate.thumbs_loaded = True
+
+        # if the collection is empty, tell the user that she can add items
+        if (not collection):
+            gobject.idle_add(dialogs.info, "No items found!",
+                      "There are no items.\n"
+                      "Please go to Media Collection in the Preferences view\n"
+                      "to tell MediaBox where to look for your files.")
             
 
     def run(self):
         """
         Runs the application.
         """
-            
+
         self.__startup()
         gtk.main()
         
