@@ -1,6 +1,6 @@
 from ui.Widget import Widget
 from ui.Label import Label
-from ui.Pixmap import Pixmap
+from ui.Pixmap import Pixmap, TEMPORARY_PIXMAP
 import theme
 
 import gtk
@@ -16,28 +16,24 @@ class TrackInfo(Widget):
     def __init__(self, esens):
     
         self.__cover = None
-        self.__buffer = Pixmap(None, 800, 400)
+        self.__buffer = TEMPORARY_PIXMAP #Pixmap(None, 758, 400)
     
         Widget.__init__(self, esens)
-        self.set_size(800, 400)
         
         self.__title = Label(esens, "-", theme.font_headline,
                              theme.color_fg_trackinfo)
         self.add(self.__title)
-        self.__title.set_pos(400, 48)
-        self.__title.set_size(400 - 48, 0)
+        self.__title.set_geometry(400, 34, 400 - 48, 0)
 
         self.__album = Label(esens, "-", theme.font_plain,
                              theme.color_fg_trackinfo)
         self.add(self.__album)
-        self.__album.set_pos(448, 108)
-        self.__album.set_size(400 - 96, 0)
+        self.__album.set_geometry(448, 94, 400 - 96, 0)
 
         self.__artist = Label(esens, "-", theme.font_plain,
                               theme.color_fg_trackinfo)
         self.add(self.__artist)
-        self.__artist.set_pos(448, 148)
-        self.__artist.set_size(400 - 96, 0)
+        self.__artist.set_geometry(448, 134, 400 - 96, 0)
         
 
     def render_this(self):
@@ -46,20 +42,23 @@ class TrackInfo(Widget):
         w, h = self.get_size()
         screen = self.get_screen()
         
-        self.__buffer.draw_subpixbuf(theme.background, 0, 0, 0, 0, w, h)        
-        self.__buffer.draw_frame(theme.viewer_music_frame, x + 45, y + 45,
-                                 _COVER_SIZE + 11, _COVER_SIZE + 11, True)
+        #self.__buffer.draw_subpixbuf(theme.background, 0, 0, 0, 0, w, h)
+        self.__buffer.fill_area(x, y, w, h, theme.color_bg)
+        self.__buffer.draw_frame(theme.viewer_music_frame,
+                                 x + 45, y + 29,
+                                 _COVER_SIZE + 11, _COVER_SIZE + 11,
+                                 True)
  
         if (self.__cover):
-            self.__buffer.draw_pixbuf(self.__cover, x + 48, y + 48)
-        else:
-            self.__buffer.fill_area(x + 48, y + 48, _COVER_SIZE, _COVER_SIZE,
-                             "#aaaaaa")
+            self.__buffer.draw_pixbuf(self.__cover, x + 48, y + 32)
+        #else:
+            #self.__buffer.fill_area(x + 48, y + 28, _COVER_SIZE, _COVER_SIZE,
+            #                        "#000000")
         
-        self.__buffer.draw_pixbuf(theme.viewer_music_album, 400, 108)
-        self.__buffer.draw_pixbuf(theme.viewer_music_artist, 400, 148)
+        self.__buffer.draw_pixbuf(theme.viewer_music_album, x + 400, y + 92)
+        self.__buffer.draw_pixbuf(theme.viewer_music_artist, x + 400, y + 132)
         
-        screen.copy_pixmap(self.__buffer, 0, 0, 0, 0, w, h)
+        screen.copy_pixmap(self.__buffer, x, y, x, y, w, h)
 
                
 
@@ -72,7 +71,7 @@ class TrackInfo(Widget):
         
         del cover
         del scaled
-        self.render()
+        #self.render()
         
         
     def set_title(self, title):
