@@ -1,5 +1,10 @@
-# the thief module tries to get existing thumbnails from the desktop or
-# other applications, so that we can skip making our own
+"""
+This module tries to get existing thumbnails from the desktop or
+other applications, so that we can skip making our own.
+If you are extending the thief's capabilities, please remember that a good
+thief needs to be quick!
+"""
+
 
 from utils import urlquote
 import os
@@ -27,7 +32,6 @@ def steal_cover(uri):
     name = basename + "_thumb.jpg"
     thumb_dir = "/media/mmc1/covers"
 
-        
     cover_uri = os.path.join(thumb_dir, name)
     if (os.path.exists(cover_uri)):
         return cover_uri
@@ -41,7 +45,7 @@ def steal_video(uri):
     basename = os.path.basename(uri)
     vid_name = basename + ".jpg"
     thumb_dir = "/media/mmc1/covers"
-        
+
     video_uri = os.path.join(thumb_dir, vid_name)
     if (os.path.exists(video_uri)):
         return video_uri
