@@ -1,21 +1,24 @@
 from Widget import Widget
-from Pixmap import Pixmap
+from Pixmap import Pixmap, TEMPORARY_PIXMAP
 
 
 class ImageButton(Widget):
 
-    def __init__(self, esens, img1, img2):
+    def __init__(self, esens, img1, img2, manual = False):
     
         self.__bg = None
         self.__buffer = None
+        self.__state = 0
         
         self.__img1 = img1
         self.__img2 = img2
     
         Widget.__init__(self, esens)
-        self.set_size(80, 80)
-        self.connect(self.EVENT_BUTTON_PRESS, self.__on_click, True)
-        self.connect(self.EVENT_BUTTON_RELEASE, self.__on_click, False)               
+        self.set_size(70, 60)
+        
+        if (not manual):
+            self.connect(self.EVENT_BUTTON_PRESS, self.__on_click, True)
+            self.connect(self.EVENT_BUTTON_RELEASE, self.__on_click, False)               
     
     
     
@@ -23,7 +26,7 @@ class ImageButton(Widget):
     
         Widget.set_size(self, w ,h)
         self.__bg = Pixmap(None, w, h)
-        self.__buffer = Pixmap(None, w, h)
+        self.__buffer = TEMPORARY_PIXMAP #Pixmap(None, w, h)
     
         
     def render_this(self):
@@ -37,7 +40,7 @@ class ImageButton(Widget):
         # save background
         self.__bg.copy_buffer(screen, x, y, 0, 0, w, h)
 
-        self.__render_button(0)            
+        self.__render_button(self.__state)            
                      
         
     def __on_click(self, px, py, clicked):
@@ -51,6 +54,7 @@ class ImageButton(Widget):
 
     def __render_button(self, state):
 
+        self.__state = state
         if (not self.may_render()): return
 
         x, y = self.get_screen_pos()
@@ -77,3 +81,11 @@ class ImageButton(Widget):
         
         self.__render_button(0)
 
+
+    def set_active(self, active):
+    
+        if (active):
+            self.__render_button(1)
+        else:
+            self.__render_button(0)
+
